<?php
/**
 * Created by PhpStorm.
 * User: matheussilva
 * Date: 08/01/2015
 * Time: 10:28
 */

namespace Folha\Logic\IntegracaoContabil;

use \Folha\Entities\Fparesumoctbl;
use \Contabilidade\Entities\Capalote;
use \Contabilidade\Entities\Lanclote;
use Geral\Entities\Estruturam;
use \HCM\Util\DateUtil;
use \Geral\Util\NovoCodigo;

class ResumoContabil {

    /**
     * DCU002 - Gerar Lote Contábil
     * Busca os resumos contábeis, gera e persiste os lotes e itens para integração.
     *
     * @param int $nrestruturam
     * @param \DateTime $dtocorrencia,
     * @param int $nrorg
     * @return boolean
     * @throws \Exception
     */
    public function resumoContabil($nrestruturam, $dtocorrencia, $nrorg)
    {
        try {
             $resumos = $this->buscarResumos($nrestruturam, $dtocorrencia, $nrorg);
            $lote = $this->geraLote($resumos[0], $nrorg);
            $itens = $this->geraItens($resumos, $lote);
            $loteInserido = $this->insereLote($lote);
            if ($loteInserido) {
                $itensInseridos = $this->insereItens($itens);
                if (!$itensInseridos) {
                    throw ExceptionsContabeis::itemLoteNaoInserido();
                }
            } else {
                throw ExceptionsContabeis::loteNaoInserido();
            }
        } catch (\Exception $e){
            throw $e;
        }

    }

    /**
     * DCU002 - Gerar Lote Contábil
     * Busca os resumos contábeis persistidos naquela competência
     *
     * @param int $nrestruturam
     * @param \DateTime $dtocorrencia
     * @param int $nrorg
     * @return Fparesumoctbl[]
     * @throws \Exception
     */
    private function buscarResumos($nrestruturam, $dtocorrencia, $nrorg)
    {
        try {
            $where = "NRESTRUTURAM = :P_NRESTRUTURAM AND DTOCORRENCIA = :P_DTOCORRENCIA AND NRORG = :P_NRORG";
            $params = array(
                'P_NRESTRUTURAM' => $nrestruturam,
                'P_NRORG' => $nrorg,
                'P_DTOCORRENCIA' => $dtocorrencia->format('d/m/Y')
            );
            return Fparesumoctbl::getObjectsByWhere($where, $params);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * DCU002 - Gerar Lote Contábil
     * Cria um objeto de lote contábil a serem persistidos
     *
     * @param Fparesumoctbl $resumo
     * @return Capalote
     * @throws \Exception
     */
    private function geraLote($resumo)
    {
        try {
            $dtMesCompetencia = DateUtil::getDataDeString(\FPA::getDtCompetencia(), DateUtil::FORMATO_BRASILEIRO, true);
            if (!$dtMesCompetencia) {
                throw ExceptionsContabeis::problemaCompetencia();
            }
            $nrtipoestrutura = $this->buscaTipoestrutura($resumo->getNrestruturam(), $resumo->getNrorg())
            $codigo = "FOLHA_HRM_{$dtMesCompetencia->format('m')}_{$dtMesCompetencia->format('Y')}_".NovoCodigo::geraCodigo($resumo->getNrorg(), 'CAPALOTE'.$dtMesCompetencia->format('dmY'),1,2, false);
            $cdempresa = (string) \Folha\Repositories\FpaResumoctbl::getCodIntEmpresa($resumo->getNrestruturam(), $nrtipoestrutura, $resumo->getNrorg());
            $observacao = "HRM Competencia {$dtMesCompetencia->format('m')}/{$dtMesCompetencia->format('Y')}";

            $lote = new Capalote();
            $lote->setCdempresa($cdempresa);
            $lote->setCdlote($codigo);
            $lote->setDtiniclote($dtMesCompetencia->format('d/m/Y'));
            $lote->setDtfinalote($dtMesCompetencia->format('t/m/Y'));
            $lote->setIdlote('D');
            $lote->setDsobselote($observacao);
            $lote->setIdrecontcaplote('P');
            $lote->setNrorg($resumo->getNrorg());
            $lote->getDtinclusao($dtMesCompetencia->format('d/m/Y'));
            $lote->setNrorginclusao($resumo->getNrorg());

            return $lote;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * DCU002 - Gerar Lote Contábil
     * Cria um array de objetos de item de lote contábil a serem persistidos
     *
     * @param Fparesumoctbl[] $resumos
     * @param Capalote $lote
     *
     * @return Lanclote[]
     * @throws \Exception
     */
    private function geraItens($resumos, Capalote $lote)
    {
        try {
            $itens = array();
            for ($c = 0; $c < count($resumos); $c++) {

                $codigo = NovoCodigo::geraCodigo($lote->getNrorg(), 'LANCLOTE' . $lote->getCdempresa(), 1, 21, true);

                $item = new Lanclote();
                $item->setCdempresa($lote->getCdempresa());
                $item->setCdlote($lote->getCdlote());
                $item->setNrlanclote($codigo);
                $item->setNrseqlanlote(str_pad($c, 5, '0', STR_PAD_LEFT));
                $item->setDtlanclote(DateUtil::getDataAtual(true)->format('d/m/Y'));
                $item->setCdcontctbl($resumos[$c]->getCdcontctbl());
                $item->setCdhistpadr($resumos[$c]->getCdhistpadr());
                $item->setVrlanclote($resumos[$c]->getVrlancamento());
                $item->setIdfcontlanclote('P');
                $item->setIdtplanclote('I');

                $itens[] = $item;
            }

            return $itens;
        } catch (\Exception $e) {
            throw $e;
        }

    }

    /**
     * DCU002 - Gerar Lote Contábil
     * Persiste o objeto de Lote Contábil no banco.
     *
     * @param Capalote $lote
     * @return boolean
     * @throws \Exception
     */
    private function insereLote(Capalote $lote)
    {
        try {
            return \Contabilidade\Repositories\Capalote::insertCapalote($lote);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * DCU002 - Gerar Lote Contábil
     * Persiste os objetos de Item de Lote Contábil no banco.
     *
     * @param Lanclote[] $item
     * @return boolean
     */
    private function insereItens($itens)
    {
        try {
            foreach ($itens as $item) {
                \Contabilidade\Repositories\Lanclote::insereLanclote($item);
            }
            return true;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Aux001
     * Busca o tipo de estrutura da estrutura legal do resumo
     * @param $nrestruturam
     * @param $nrorg
     * @return int
     */
    private function buscaTipoestrutura($nrestruturam, $nrorg)
    {
        $params = array(
            'P_NRESTRUTURAM' => $nrestruturam,
            'P_NRORG' => $nrorg
        );
        $where = "NRESTRUTURAM = :P_NRESTRUTURAM AND NRORG = :P_NRORG";
        $estruturam = Estruturam::getObjectByWhere($where, $params);

        return $estruturam->getNrtipoestrutura();
    }

} 